@@ -190,13 +190,14 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
-      gal_list_str_add(&p->inputnames, arg, 0);
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
+      if(arg[0]!='\0') gal_list_str_add(&p->inputnames, arg, 0);
       break;
 
-
     /* This is an option, set its value. */
     default:
       return gal_options_set_from_key(key, arg, p->cp.poptions, &p->cp);
@@ -235,14 +236,17 @@ ui_colormap_sanity_check(struct converttparams *p)
   size_t nparams=0;
   int ccode=COLOR_INVALID;
 
-  /* See how many parameters are necessary. */
+  /* See how many parameters are necessary.
+     Notes for TAB completion:
+        1. Keep 'gray' and 'grey' in the same line.
+        2. Keep a space after the ',' before the strings.   */
   strarr=p->colormap->array;
-  if     ( !strcmp(strarr[0],"hsv"))     { ccode=COLOR_HSV;     nparams=2; }
-  else if( !strcmp(strarr[0],"sls"))     { ccode=COLOR_SLS;     nparams=0; }
-  else if( !strcmp(strarr[0],"viridis")) { ccode=COLOR_VIRIDIS; nparams=0; }
-  else if( !strcmp(strarr[0],"gray") || !strcmp(strarr[0],"grey"))
-                                      { ccode=COLOR_GRAY; nparams=0; }
-  else if( !strcmp(strarr[0],"sls-inverse"))
+  if     ( !strcmp(strarr[0], "hsv"))     { ccode=COLOR_HSV;     nparams=2; }
+  else if( !strcmp(strarr[0], "sls"))     { ccode=COLOR_SLS;     nparams=0; }
+  else if( !strcmp(strarr[0], "viridis")) { ccode=COLOR_VIRIDIS; nparams=0; }
+  else if( !strcmp(strarr[0], "gray") || !strcmp(strarr[0], "grey"))
+                                         { ccode=COLOR_GRAY; nparams=0; }
+  else if( !strcmp(strarr[0], "sls-inverse"))
     { ccode=COLOR_SLS_INVERSE; nparams=0; }
   else
     error(EXIT_FAILURE, 0, "'%s' not recognized as a colormap given "
@@ -337,7 +341,8 @@ ui_read_check_only_options(struct converttparams *p)
 
   if(p->fluxhighstr && p->fluxlowstr)
     {
-      cond=gal_arithmetic(GAL_ARITHMETIC_OP_GT, 1, GAL_ARITHMETIC_NUMOK,
+      cond=gal_arithmetic(GAL_ARITHMETIC_OP_GT, 1,
+                          GAL_ARITHMETIC_FLAG_NUMOK,
                           p->fluxhigh, p->fluxlow);
 
       if( *((unsigned char *)cond->array) == 0 )
@@ -515,20 +520,30 @@ ui_make_channels_ll(struct converttparams *p)
       if(strcmp(name->v, "blank")) gal_checkset_check_file(name->v);
 
       /* FITS: */
-      if( gal_fits_name_is_fits(name->v) )
+      if( gal_fits_file_recognized(name->v) )
         {
           /* Get the HDU value for this channel. */
-          if(p->hdus)
-            hdu=gal_list_str_pop(&p->hdus);
+          if(p->globalhdu)
+            hdu=p->globalhdu;
           else
-            error(EXIT_FAILURE, 0, "not enough HDUs. Every input FITS image "
-                  "needs a HDU, you can use the '--hdu' ('-h') option once "
-                  "for each input FITS image (in the same order)");
+            {
+              if(p->hdus)
+                hdu=gal_list_str_pop(&p->hdus);
+              else
+                error(EXIT_FAILURE, 0, "not enough HDUs. Every input FITS "
+                      "image needs a HDU (identified by name or number, "
+                      "counting from zero). You can use multiple calls to "
+                      "the '--hdu' ('-h') option for each input FITS image "
+                      "(in the same order as the input FITS files), or use "
+                      "'--globalhdu' ('-g') once when the same HDU should "
+                      "be used for all of them");
+            }
 
           /* Read in the array and its WCS information. */
           data=gal_fits_img_read(name->v, hdu, p->cp.minmapsize,
                                  p->cp.quietmmap);
-          data->wcs=gal_wcs_read(name->v, hdu, 0, 0, &data->nwcs);
+          data->wcs=gal_wcs_read(name->v, hdu, p->cp.wcslinearmatrix,
+                                 0, 0, &data->nwcs);
           data->ndim=gal_dimension_remove_extra(data->ndim, data->dsize,
                                                 data->wcs);
           gal_list_data_add(&p->chll, data);
@@ -617,6 +632,97 @@ ui_make_channels_ll(struct converttparams *p)
 
 
 
+static void
+ui_prepare_input_channels_check_wcs(struct converttparams *p)
+{
+  int printwarning=0;
+  float wcsmatch=1.0;
+  gal_data_t *tmp, *coords=NULL;
+  size_t one=1, numwcs=0, numnonblank=0;
+  double *c1, *c2, r1=NAN, r2=NAN, *pixscale=NULL;
+
+  /* If all the inputs have WCS, check to see if the inputs are aligned and
+     print a warning if they aren't. */
+  for(tmp=p->chll; tmp!=NULL; tmp=tmp->next)
+    {
+      if(tmp->wcs && tmp->type!=GAL_TYPE_INVALID) ++numwcs;
+      if(tmp->type!=GAL_TYPE_INVALID)             ++numnonblank;
+    }
+  if(numwcs==numnonblank)
+    {
+      /* Allocate the coordinate columns. */
+      gal_list_data_add_alloc(&coords, NULL, GAL_TYPE_FLOAT64, 1,
+                              &one, NULL, 0, -1, 1, NULL, NULL, NULL);
+      gal_list_data_add_alloc(&coords, NULL, GAL_TYPE_FLOAT64, 1,
+                              &one, NULL, 0, -1, 1, NULL, NULL, NULL);
+
+      /* Go over each image and put its central pixel in the coordinates
+         and do the world-coordinate transformation. Recall that the C
+         coordinates are the inverse order of FITS coordinates and that
+         FITS coordinates count from 1 (not 0).*/
+      for(tmp=p->chll; tmp!=NULL; tmp=tmp->next)
+        if(tmp->wcs)
+          {
+            /* Fill the coordinate values. */
+            c1=coords->array;
+            c2=coords->next->array;
+            c1[0] = tmp->dsize[1] / 2 + 1;
+            c2[0] = tmp->dsize[0] / 2 + 1;
+
+            /* Get the RA/Dec. */
+            gal_wcs_img_to_world(coords, tmp->wcs, 1);
+
+            /* If the pixel scale hasn't been calculated yet, do it (we
+               only need it once, should be similar in all). */
+            if(pixscale==NULL)
+              pixscale=gal_wcs_pixel_scale(tmp->wcs);
+
+            /* If the reference/first center is not yet defined then write
+               the conversions in it. If it is defined, compare with it
+               with the new dataset and print a warning if necessary. */
+            if( isnan(r1) )
+              { r1=c1[0]; r2=c2[0]; }
+            else
+              {
+                /* For a check.
+                printf("check: %g, %g\n", fabs(c1[0]-r1)/pixscale[0],
+                       fabs(c2[0]-r2)/pixscale[1]);
+                */
+
+                /* See if a warning should be printed. */
+                if( fabs(c1[0]-r1)/pixscale[0] > wcsmatch
+                    || fabs(c2[0]-r2)/pixscale[1] > wcsmatch )
+                  printwarning=1;
+              }
+          }
+    }
+
+  /* Print the warning message if necessary. */
+  if(printwarning && p->cp.quiet==0)
+    {
+      error(EXIT_SUCCESS, 0, "WARNING: The WCS information of the input "
+            "FITS images don't match (by more than %g pixels in the "
+            "center), even though the input images have the same number "
+            "of pixels in each dimension. Therefore the color channels "
+            "of the output colored image may not be aligned. If this is "
+            "not a problem, you can suppress this warning with the "
+            "'--quiet' option.\n\n"
+            "A solution to align your images is provided in the "
+            "\"Aligning images with small WCS offsets\" section of "
+            "Gnuastro's manual. Please run the command below to see "
+            "it (you can return to the command-line by pressing 'q'):\n\n"
+            "   info gnuastro \"Aligning images\"\n",
+            wcsmatch);
+    }
+
+  /* Clean up. */
+  free(pixscale);
+}
+
+
+
+
+
 /* Read the input(s)/channels. */
 static void
 ui_prepare_input_channels(struct converttparams *p)
@@ -692,6 +798,8 @@ ui_prepare_input_channels(struct converttparams *p)
           wcs=tmp->wcs;
       }
 
+  /* Make sure the images are all aligned to the same grid. */
+  ui_prepare_input_channels_check_wcs(p);
 
   /* If ndim is still NULL, then there were no non-blank inputs, so print
      an error. */
