@@ -165,13 +165,15 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
       if(p->filename)
         argp_error(state, "only one argument (input file) should be given");
       else
-        p->filename=arg;
+        if(arg[0]!='\0') p->filename=arg;
       break;
 
 
@@ -252,7 +254,7 @@ ui_check_options_and_arguments(struct convolveparams *p)
   if(p->filename)
     {
       /* If input is FITS. */
-      if( (p->isfits=gal_fits_name_is_fits(p->filename)) )
+      if( (p->isfits=gal_fits_file_recognized(p->filename)) )
         {
           /* Check if a HDU is given. */
           if( p->cp.hdu==NULL )
@@ -275,7 +277,7 @@ ui_check_options_and_arguments(struct convolveparams *p)
   if(p->kernelname)
     {
       /* If input is FITS. */
-      if( gal_fits_name_is_fits(p->kernelname) )
+      if( gal_fits_file_recognized(p->kernelname) )
         {
           /* Check if a HDU is given. */
           if( p->khdu==NULL )
@@ -438,7 +440,8 @@ ui_read_input(struct convolveparams *p)
                                                INPUT_USE_TYPE,
                                                p->cp.minmapsize,
                                                p->cp.quietmmap);
-        p->input->wcs=gal_wcs_read(p->filename, p->cp.hdu, 0, 0,
+        p->input->wcs=gal_wcs_read(p->filename, p->cp.hdu,
+                                   p->cp.wcslinearmatrix, 0, 0,
                                    &p->input->nwcs);
         p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
                                                   p->input->dsize,
@@ -579,11 +582,13 @@ ui_preparations(struct convolveparams *p)
           sum=gal_statistics_sum(p->input);
           sum=gal_data_copy_to_new_type_free(sum, GAL_TYPE_FLOAT32);
           p->input = gal_arithmetic(GAL_ARITHMETIC_OP_DIVIDE, 1,
-                                    GAL_ARITHMETIC_FLAGS_ALL, p->input, sum);
+                                    GAL_ARITHMETIC_FLAGS_BASIC, p->input,
+                                    sum);
           sum=gal_statistics_sum(p->kernel);
           sum=gal_data_copy_to_new_type_free(sum, GAL_TYPE_FLOAT32);
           p->kernel = gal_arithmetic(GAL_ARITHMETIC_OP_DIVIDE, 1,
-                                     GAL_ARITHMETIC_FLAGS_ALL, p->kernel, sum);
+                                     GAL_ARITHMETIC_FLAGS_BASIC,
+                                     p->kernel, sum);
         }
     }
 
