@@ -120,7 +120,7 @@ ui_initialize_options(struct fitsparams *p,
         case GAL_OPTIONS_KEY_SEARCHIN:
         case GAL_OPTIONS_KEY_IGNORECASE:
         case GAL_OPTIONS_KEY_TYPE:
-        case GAL_OPTIONS_KEY_TABLEFORMAT:
+        case GAL_OPTIONS_KEY_WCSLINEARMATRIX:
         case GAL_OPTIONS_KEY_DONTDELETE:
         case GAL_OPTIONS_KEY_LOG:
         case GAL_OPTIONS_KEY_NUMTHREADS:
@@ -172,19 +172,18 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
-    /* Read the non-option tokens (arguments): */
+    /* Read the non-option tokens (arguments). */
     case ARGP_KEY_ARG:
-      /* Only FITS files are acceptable. */
-      if( gal_fits_name_is_fits(arg) )
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
+      if(arg[0]!='\0')
         {
-          if(p->filename)
-            argp_error(state, "only one input file should be given");
+          if( gal_fits_file_recognized(arg) )
+            gal_list_str_add(&p->input, arg, 1);
           else
-            p->filename=arg;
+            argp_error(state, "%s is not a recognized FITS file", arg);
         }
-      else
-        argp_error(state, "%s is not a recognized FITS file", arg);
       break;
 
 
@@ -322,13 +321,15 @@ ui_check_copykeys(struct fitsparams *p)
 static void
 ui_read_check_only_options(struct fitsparams *p)
 {
+  int checkkeys;
   uint8_t stdoutcheck=0;
 
   /* If any of the keyword manipulation options are requested, then set the
      mode flag to keyword-mode. */
-  if( p->date || p->comment || p->history || p->asis || p->delete
-      || p->rename || p->update || p->write || p->verify || p->printallkeys
-      || p->copykeys || p->datetosec || p->wcsdistortion )
+  if( p->date || p->comment || p->history || p->asis || p->keyvalue
+      || p->delete || p->rename || p->update || p->write || p->verify
+      || p->printallkeys || p->printkeynames || p->copykeys || p->datetosec
+      || p->wcscoordsys || p->wcsdistortion )
     {
       /* Check if a HDU is given. */
       if(p->cp.hdu==NULL)
@@ -340,30 +341,44 @@ ui_read_check_only_options(struct fitsparams *p)
       if(p->copykeys)
         ui_check_copykeys(p);
 
-      /* Currently 'datetosec' must be called alone. */
-      if( (p->datetosec || p->wcsdistortion)
-          && ( (p->datetosec && p->wcsdistortion)
-               || p->date || p->comment || p->history || p->asis || p->delete
-               || p->rename || p->update || p->write || p->verify
-               || p->printallkeys || p->copykeys
-               )
-          )
-        error(EXIT_FAILURE, 0, "'--datetosec' and '--wcsdistortion' cannot "
+      /* Keyword-related options that must be called alone. */
+      checkkeys = ( (p->keyvalue!=NULL)
+                    + (p->datetosec!=NULL)
+                    + (p->printkeynames!=0)
+                    + (p->wcscoordsys!=NULL)
+                    + (p->wcsdistortion!=NULL) );
+      if( ( checkkeys
+            && ( p->date || p->comment || p->history || p->asis
+                 || p->delete || p->rename || p->update || p->write
+                 || p->verify || p->printallkeys || p->copykeys ) )
+          || checkkeys>1 )
+        error(EXIT_FAILURE, 0, "'--keyvalue', '--datetosec', "
+              "'--wcscoordsys' and '--wcsdistortion' cannot "
               "currently be called with any other option");
 
+      /* Give an ID to recognized coordinate systems. */
+      if(p->wcscoordsys)
+        p->coordsysid=gal_wcs_coordsys_from_string(p->wcscoordsys);
+
       /* Identify the requested distortion. Note that this also acts as a
          sanity check because it will crash with an error if the given
          string isn't recognized. */
       if(p->wcsdistortion)
         p->distortionid=gal_wcs_distortion_from_string(p->wcsdistortion);
 
+      /* Make sure the value of '--keyvalue' is actually present. */
+      if(p->keyvalue && p->keyvalue->v[0]=='\0')
+        error(EXIT_FAILURE, 0, "the '--keyvalue' option requires a value: "
+              "the name(s) of keywords you want the value of");
+
       /* Set the operating mode. */
       p->mode=FITS_MODE_KEY;
     }
 
   /* Same for the extension-related options */
   if( p->remove || p->copy || p->cut || p->numhdus || p->datasum
-      || p->pixelscale || p->skycoverage )
+      || p->pixelscale || p->skycoverage || p->hastablehdu
+      || p->hasimagehdu || p->listtablehdus || p->listimagehdus )
     {
       /* A small sanity check. */
       if(p->mode!=FITS_MODE_INVALID)
@@ -371,26 +386,32 @@ ui_read_check_only_options(struct fitsparams *p)
               "cannot be called together");
 
       /* Some HDU options cannot be called with other options. */
-      stdoutcheck = p->numhdus + p->datasum + p->pixelscale + p->skycoverage;
+      stdoutcheck = ( p->numhdus + p->datasum + p->pixelscale
+                      + p->skycoverage + p->hastablehdu + p->hasimagehdu
+                      + p->listtablehdus + p->listimagehdus );
 
       /* Make sure if an output file is needed. */
       if(stdoutcheck)
         {
-          /* Make sure the other HDU-related options aren't called. */
+          /* Make sure HDU reading and editing options aren't called
+             together. */
           if(p->remove || p->copy || p->cut)
-            error(EXIT_FAILURE, 0, "'--numhdus', '--datasum', '--pixelscale' "
-                  "or '--skycoverage' options cannot be called with any of "
-                  "the '--remove', '--copy' or '--cut' options");
+            error(EXIT_FAILURE, 0, "HDU reading options (like "
+                  "'--numhdus', '--datasum' and etc) cannot be called "
+                  "with any of the HDU modification options like "
+                  "'--remove', '--copy' or '--cut' options");
 
           /* Make sure these options are called alone. */
           if(stdoutcheck>1)
-            error(EXIT_FAILURE, 0, "'--numhdus', '--datasum', '--pixelscale' "
-                  "or '--skycoverage' options cannot be called together, "
-                  "only one at a time");
+            error(EXIT_FAILURE, 0, "HDU info options, like '--numhdus', "
+                  "'--datasum', '--pixelscale' or '--skycoverage', cannot "
+                  "be called together, only one at a time");
 
           /* Make sure the HDU is given if any of the options except
              '--numhdus' are called. */
-          if( stdoutcheck-p->numhdus && p->cp.hdu==NULL )
+          if( ( p->numhdus || p->hastablehdu || p->hasimagehdu
+                || p->listtablehdus || p->listimagehdus)
+              && p->cp.hdu==NULL )
             error(EXIT_FAILURE, 0, "a HDU (extension) is necessary for the "
                   "'--datasum', '--pixelscale' or '--skycoverage' options. "
                   "Please use the '--hdu' (or '-h') option to select one");
@@ -400,7 +421,7 @@ ui_read_check_only_options(struct fitsparams *p)
           if(p->cp.output)
             gal_checkset_writable_remove(p->cp.output, 1, p->cp.dontdelete);
           else
-            p->cp.output=gal_checkset_automatic_output(&p->cp, p->filename,
+            p->cp.output=gal_checkset_automatic_output(&p->cp, p->input->v,
                                                        "_ext.fits");
         }
 
@@ -431,8 +452,15 @@ ui_check_options_and_arguments(struct fitsparams *p)
 {
   /* Make sure an input file name was given and if it was a FITS file, that
      a HDU is also given. */
-  if(p->filename==NULL)
+  if(p->input==NULL)
     error(EXIT_FAILURE, 0, "no input file is specified");
+  gal_list_str_reverse(&p->input);
+
+  /* More than one input is currently only acceptable with the '--keyvalue'
+     option. */
+  if( gal_list_str_number(p->input) > 1 && p->keyvalue==NULL)
+    error(EXIT_FAILURE, 0, "one input file is expected but %zu input "
+          "files are given", gal_list_str_number(p->input));
 }
 
 
