@@ -187,16 +187,17 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
       if(p->objectsfile)
         argp_error(state, "only one argument (input file) should be given");
       else
-        p->objectsfile=arg;
+        if(arg[0]!='\0') p->objectsfile=arg;
       break;
 
-
     /* This is an option, set its value. */
     default:
       return gal_options_set_from_key(key, arg, p->cp.poptions, &p->cp);
@@ -454,7 +455,7 @@ ui_check_options_and_arguments(struct mkcatalogparams *p)
      and if it was a FITS file, that a HDU is also given. */
   if(p->objectsfile)
     {
-      if( gal_fits_name_is_fits(p->objectsfile) && p->cp.hdu==NULL )
+      if( gal_fits_file_recognized(p->objectsfile) && p->cp.hdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified. When the input is a FITS "
               "file, a HDU must also be specified, you can use the '--hdu' "
               "('-h') option and give it the HDU number (starting from "
@@ -538,7 +539,8 @@ ui_wcs_info(struct mkcatalogparams *p)
   size_t i;
 
   /* Read the WCS meta-data. */
-  p->objects->wcs=gal_wcs_read(p->objectsfile, p->cp.hdu, 0, 0,
+  p->objects->wcs=gal_wcs_read(p->objectsfile, p->cp.hdu,
+                               p->cp.wcslinearmatrix, 0, 0,
                                &p->objects->nwcs);
 
   /* Read the basic WCS information. */
@@ -1388,9 +1390,7 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
           zero=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &one, NULL, 1, -1,
                               1, NULL, NULL, NULL);
           p->upmask=gal_arithmetic(GAL_ARITHMETIC_OP_NE, 1,
-                                   ( GAL_ARITHMETIC_INPLACE
-                                     | GAL_ARITHMETIC_FREE
-                                     | GAL_ARITHMETIC_NUMOK ),
+                                   GAL_ARITHMETIC_FLAGS_BASIC,
                                    p->upmask, zero);
         }
     }
@@ -1423,10 +1423,20 @@ ui_preparations_read_keywords(struct mkcatalogparams *p)
           keys[0].array=&minstd;              keys[1].array=&p->medstd;
           gal_fits_key_read(p->usedstdfile, p->stdhdu, keys, 0, 0);
 
-          /* If the two keywords couldn't be read. We don't want to slow down
-             the user for the median (which needs sorting). So we'll just
-             calculate the minimum which is necessary for the 'p->cpscorr'. */
-          if(keys[1].status) p->medstd=NAN;
+          /* If the two keywords couldn't be read. We don't want to slow
+             down the user for the median (which needs sorting). So we'll
+             just calculate it if '--forcereadstd' is called. However, we
+             need the minimum for 'p->cpscorr'. */
+          if(keys[1].status)
+            {
+              if(p->forcereadstd)
+                {
+                  tmp=gal_statistics_median(p->std, 0);
+                  p->medstd=*((float *)(tmp->array));
+                }
+              else
+                p->medstd=NAN;
+            }
           if(keys[0].status)
             {
               /* Calculate the minimum STD. */
