@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -124,6 +124,9 @@ ui_profile_name_read(char *string, size_t row)
   else if ( !strcmp("distance", string) )
     return PROFILE_DISTANCE;
 
+  else if ( !strcmp("custom", string) )
+    return PROFILE_CUSTOM;
+
   else if ( !strcmp(GAL_BLANK_STRING, string) )
     error(EXIT_FAILURE, 0, "atleast one profile function is blank");
 
@@ -187,6 +190,9 @@ ui_initialize_options(struct mkprofparams *p,
   cp->numthreads         = gal_threads_number();
   cp->coptions           = gal_commonopts_options;
 
+  p->customregular[0]    = NAN;
+  p->customregular[1]    = NAN;
+
   /* Default program parameters. */
   p->zeropoint           = NAN;
   p->cp.type             = GAL_TYPE_FLOAT32;
@@ -250,16 +256,17 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
       if(p->catname)
         argp_error(state, "only one argument (input catalog) may be given");
       else
-        p->catname=arg;
+        if(arg[0]!='\0') p->catname=arg;
       break;
 
-
     /* This is an option, set its value. */
     default:
       return gal_options_set_from_key(key, arg, p->cp.poptions, &p->cp);
@@ -326,6 +333,10 @@ ui_parse_kernel(struct argp_option *option, char *arg,
     }
   else
     {
+      /* If the kernel has already been given, ignore it (the previously
+         read value has higher precedence). */
+      if( *(gal_data_t **)(option->value) ) return NULL;
+
       /* The first part of 'arg' (before the first comma) is not
          necessarily a number. So we need to separate the first part from
          the rest.*/
@@ -334,9 +345,18 @@ ui_parse_kernel(struct argp_option *option, char *arg,
       arg = (*c=='\0') ? NULL : c+1;  /* the 'point' profile doesn't need */
       *c='\0';                        /* any numbers.                     */
 
+      /* Make sure something exists after the name of the profile. */
+      if(arg==NULL)
+        error(EXIT_FAILURE, 0, "the kernel option value couldn't be "
+              "parsed in the expected format: one name (of a profile), "
+              "followed by some numbers defining that profile. See the "
+              "description of '--kernel' in the manual (with the 'info "
+              "astmkprof' command) for the meaning of the numbers");
 
       /* Read the parameters. */
       kernel=gal_options_parse_list_of_numbers(arg, filename, lineno);
+
+      /* Put the kernel dataset into the main program structure. */
       *(gal_data_t **)(option->value) = kernel;
 
 
@@ -559,7 +579,7 @@ ui_check_options_and_arguments(struct mkprofparams *p)
     {
       if(p->catname)
         {
-          if( gal_fits_name_is_fits(p->catname) && p->cp.hdu==NULL)
+          if( gal_fits_file_recognized(p->catname) && p->cp.hdu==NULL)
             error(EXIT_FAILURE, 0, "no 'hdu' specified for the input FITS "
                   "table '%s', to ", p->catname);
         }
@@ -1093,6 +1113,7 @@ ui_read_cols_3d(struct mkprofparams *p)
 static void
 ui_prepare_columns(struct mkprofparams *p)
 {
+  size_t i;
   double *karr;
   float r, n, t, q2;
 
@@ -1191,6 +1212,21 @@ ui_prepare_columns(struct mkprofparams *p)
                 __func__, PACKAGE_BUGREPORT, p->ndim);
         }
     }
+
+  /* If a custom profile is requested, make sure that a custom file is
+     given. */
+  for(i=0;i<p->num;++i)
+    if(p->f[i]==PROFILE_CUSTOM)
+      {
+        if(p->customname==NULL)
+          error(EXIT_FAILURE, 0, "at least one custom profile requested "
+                "(first occurrence in row %zu), but no file/table was given "
+                "to the '--customtable' option. See the description of "
+                "this '--customtable' for more information on the "
+                "desired format", i+1);
+        break;
+      }
+
 }
 
 
@@ -1324,6 +1360,10 @@ ui_prepare_wcs(struct mkprofparams *p)
   if(status)
     error(EXIT_FAILURE, 0, "wcsset error %d: %s", status,
           wcs_errmsg[status]);
+
+  /* Convert it to CD if the user wanted it. */
+  if(p->cp.wcslinearmatrix==GAL_WCS_LINEAR_MATRIX_CD)
+    gal_wcs_to_cd(wcs);
 }
 
 
@@ -1349,7 +1389,8 @@ ui_prepare_canvas(struct mkprofparams *p)
          the background image and the number of its dimensions. So
          'ndim==0' and what 'dsize' points to is irrelevant. */
       tdsize=gal_fits_img_info_dim(p->backname, p->backhdu, &tndim);
-      p->wcs=gal_wcs_read(p->backname, p->backhdu, 0, 0, &p->nwcs);
+      p->wcs=gal_wcs_read(p->backname, p->backhdu, p->cp.wcslinearmatrix,
+                          0, 0, &p->nwcs);
       tndim=gal_dimension_remove_extra(tndim, tdsize, p->wcs);
       free(tdsize);
       if(p->nomerged==0)
@@ -1637,6 +1678,83 @@ ui_make_log(struct mkprofparams *p)
 
 
 
+/* Read the input radial table. */
+static void
+ui_read_custom_table(struct mkprofparams *p)
+{
+  size_t i;
+  double diff;
+  int isregular;
+  gal_data_t *cols;
+  double *min, *max;
+
+  /* Read the input radial table. */
+  cols=gal_table_read(p->customname, p->customhdu,
+                      NULL, NULL, p->cp.searchin, p->cp.ignorecase,
+                      p->cp.minmapsize, p->cp.quietmmap, NULL);
+
+  /* Make sure the table only has three columns. */
+  if(gal_list_data_number(cols) != 3 )
+    error(EXIT_FAILURE, 0, "%s: has %zu columns, but it should only have "
+          "three columns. Column1: the radial interval's lower value. "
+          "Column 2: the radial interval's higher value. Column 3: the value "
+          "to use for pixels within that radius interval",
+          gal_fits_name_save_as_string(p->customname, p->customhdu),
+          gal_list_data_number(cols));
+
+  /* Make sure none of the three columns are string type. */
+  if( cols->type==GAL_TYPE_STRING
+      || cols->next->type==GAL_TYPE_STRING
+      || cols->next->next->type==GAL_TYPE_STRING )
+    error(EXIT_FAILURE, 0, "%s: the columns should only have numeric "
+          "data types", gal_fits_name_save_as_string(p->customname,
+                                                     p->customhdu));
+
+  /* Fill the final table as a double type. */
+  p->custom=gal_data_copy_to_new_type(cols, GAL_TYPE_FLOAT64);
+  p->custom->next=gal_data_copy_to_new_type(cols->next,
+                                                 GAL_TYPE_FLOAT64);
+  p->custom->next->next=gal_data_copy_to_new_type(cols->next->next,
+                                                       GAL_TYPE_FLOAT64);
+
+  /* Make sure the first column values are smaller than the second column's
+     values. */
+  min=p->custom->array;
+  max=p->custom->next->array;
+  for(i=0;i<p->custom->size;++i)
+    if(min[i]>=max[i])
+      error(EXIT_FAILURE, 0, "%s: the first column of row %zu (with value "
+            "%g) is larger or equal to the second column (with value %g). "
+            "However, the first column is the lower-limit of the radial "
+            "interval and the second column is the upper-limit. So the "
+            "first column must have a lower value",
+            gal_fits_name_save_as_string(p->customname,
+                                         p->customhdu), i+1,
+            min[i], max[i]);
+
+  /* Check if the input table is regular and sorted (which can greatly
+     speed up its usage). */
+  isregular=1;
+  diff=max[0]-min[0];
+  for(i=1;i<p->custom->size;++i)
+    if( min[i]<min[i-1]
+        || min[i] != max[i-1]
+        || max[i]-min[i] != diff )
+      isregular=0;
+  if(isregular)
+    {
+      p->customregular[0]=min[0];
+      p->customregular[1]=diff;
+    }
+
+  /* Clean up. */
+  gal_list_data_free(cols);
+}
+
+
+
+
+
 static void
 ui_read_ndim(struct mkprofparams *p)
 {
@@ -1721,6 +1839,9 @@ ui_preparations(struct mkprofparams *p)
      build the profiles). */
   ui_prepare_columns(p);
 
+  /* Read the radial table. */
+  if(p->customname) ui_read_custom_table(p);
+
   /* If the kernel option was given, some parameters need to be
      over-written: */
   if(p->kernel)
