@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2019, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -53,6 +53,33 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /****************************************************************
  ************           Initial detection            ************
  ****************************************************************/
+/* The user gives connectivity values in terms of the number of neighbors
+   to use, so with this function, we convert that number to the
+   connectivity that is defined in `gnuastro/dimension.h'. */
+static int
+detection_ngb_to_connectivity(size_t ndim, size_t ngb)
+{
+  int connectivity=0;
+  switch(ndim)
+    {
+    case 2:
+      connectivity = ngb==4 ? 1 : 2;
+      break;
+    case 3:
+      connectivity = ngb==6 ? 1 : (ngb==18 ? 2 : 3);
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to address "
+            "the problem. %zu not a valid value to `ndim'",
+            __func__, PACKAGE_BUGREPORT, ndim);
+    }
+  return connectivity;
+}
+
+
+
+
+
 void
 detection_initial(struct noisechiselparams *p)
 {
@@ -92,7 +119,9 @@ detection_initial(struct noisechiselparams *p)
 
   /* Erode the image. */
   if(!p->cp.quiet) gettimeofday(&t1, NULL);
-  gal_binary_erode(p->binary, p->erode, p->erodengb==4 ? 1 : 2, 1);
+  gal_binary_erode(p->binary, p->erode,
+                   detection_ngb_to_connectivity(p->input->ndim,
+                                                 p->erodengb), 1);
   if(!p->cp.quiet)
     {
       if( asprintf(&msg, "Eroded %zu time%s (%zu-connected).", p->erode,
@@ -116,7 +145,9 @@ detection_initial(struct noisechiselparams *p)
 
   /* Do the opening. */
   if(!p->cp.quiet) gettimeofday(&t1, NULL);
-  gal_binary_open(p->binary, p->opening, p->openingngb==4 ? 1 : 2, 1);
+  gal_binary_open(p->binary, p->opening,
+                  detection_ngb_to_connectivity(p->input->ndim,
+                                                p->openingngb), 1);
   if(!p->cp.quiet)
     {
       if( asprintf(&msg, "Opened (depth: %zu, %zu-connected).",
@@ -141,7 +172,7 @@ detection_initial(struct noisechiselparams *p)
                                                     p->binary->ndim);
   if(p->detectionname)
     {
-      p->olabel->name="OPENED_AND_LABELED";
+      p->olabel->name="OPENED-AND-LABELED";
       gal_fits_img_write(p->olabel, p->detectionname, NULL, PROGRAM_NAME);
       p->olabel->name=NULL;
     }
@@ -265,7 +296,7 @@ detection_fill_holes_open(void *in_prm)
       tile->block=fho_prm->workbin;
 
       /* Copy the tile into the contiguous patch of memory to work on, but
-         first reset the size element so `gal_data_copy_to_allocated' knows
+         first reset the size element so 'gal_data_copy_to_allocated' knows
          there is enough space. */
       copy->flag=0;
       copy->size=p->maxltcontig;
@@ -283,7 +314,9 @@ detection_fill_holes_open(void *in_prm)
 
       /* Fill the holes in this tile: holes with maximal connectivity means
          that they are most strongly bounded. */
-      gal_binary_holes_fill(copy, p->holengb==4 ? 1 : 2, -1);
+      gal_binary_holes_fill(copy, detection_ngb_to_connectivity(p->input->ndim,
+                                                                p->holengb),
+                            -1);
       if(fho_prm->step==1)
         {
           detection_write_in_large(tile, copy);
@@ -293,7 +326,9 @@ detection_fill_holes_open(void *in_prm)
         }
 
       /* Open all the regions. */
-      gal_binary_open(copy, p->dopening, p->dopeningngb==4 ? 1 : 2, 1);
+      gal_binary_open(copy, p->dopening,
+                      detection_ngb_to_connectivity(p->input->ndim,
+                                                    p->dopeningngb), 1);
 
       /* Write the copied region back into the large input and AFTERWARDS,
          correct the tile's pointers, the pointers must not be corrected
@@ -327,8 +362,8 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
   float *f;
   uint8_t *b, *bf;
   gal_data_t *bin;
-  int con=p->pseudoconcomp==4 ? 1 : 2;
   struct fho_params fho_prm={0, NULL, workbin, worklab, p};
+  int con=detection_ngb_to_connectivity(p->input->ndim, p->pseudoconcomp);
 
 
   /* Set all the initial detected pixels to blank values. */
@@ -343,8 +378,8 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
 
   /* Allocate the space necessary to work on each tile (to avoid having to
      allocate it it separately for each tile and within each
-     thread. `maxltcontig' is the maximum contiguous patch of memory needed
-     to store all tiles. Finally, since we are working on a `uint8_t' type,
+     thread. 'maxltcontig' is the maximum contiguous patch of memory needed
+     to store all tiles. Finally, since we are working on a 'uint8_t' type,
      the size of each element is only 1 byte. */
   fho_prm.copyspace=gal_pointer_allocate(GAL_TYPE_UINT8,
                                          p->cp.numthreads*p->maxltcontig, 0,
@@ -365,7 +400,7 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
       /* Do each step. */
       while(fho_prm.step<3)
         {
-          /* Put a copy of `workbin' into `bin' for every step (only
+          /* Put a copy of 'workbin' into 'bin' for every step (only
              necessary for the second step and after). For the first time
              it was already copied.*/
           if(fho_prm.step>1)
@@ -373,7 +408,8 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
 
           /* Do the respective step. */
           gal_threads_spin_off(detection_fill_holes_open, &fho_prm,
-                               p->ltl.tottiles, p->cp.numthreads);
+                               p->ltl.tottiles, p->cp.numthreads,
+                               p->cp.minmapsize, p->cp.quietmmap);
 
           /* Reset the blank values (if they were changed). */
           if( p->blankasforeground==0 && gal_blank_present(p->input,0) )
@@ -405,12 +441,12 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
           ++fho_prm.step;
         }
 
-      /* Clean up: the array in `bin' should just be replaced with that in
-         `workbin' because it is used in later steps. */
+      /* Clean up: the array in 'bin' should just be replaced with that in
+         'workbin' because it is used in later steps. */
       if(workbin->mmapname)
         {
-          /* Delete the memory mapped file and set the filename of `bin'
-             for `workbin'. */
+          /* Delete the memory mapped file and set the filename of 'bin'
+             for 'workbin'. */
           remove(workbin->mmapname);
           free(workbin->mmapname);
           workbin->mmapname=bin->mmapname;
@@ -424,14 +460,15 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
     }
   else
     gal_threads_spin_off(detection_fill_holes_open, &fho_prm,
-                         p->ltl.tottiles, p->cp.numthreads);
+                         p->ltl.tottiles, p->cp.numthreads,
+                         p->cp.minmapsize, p->cp.quietmmap);
 
   /* Clean up. */
   free(fho_prm.copyspace);
 
 
   /* Label all regions, but first, deal with the blank pixels in the
-     `workbin' dataset when working on the Sky. Recall that in this case,
+     'workbin' dataset when working on the Sky. Recall that in this case,
      the blank pixels are the detections. On the Sky image, blank should be
      set to 1 (because we want the detected objects to have the same labels
      as the pseudo-detections that cover them). This will allow us to later
@@ -458,8 +495,8 @@ detection_sn_write_to_file(struct noisechiselparams *p, gal_data_t *sn,
   gal_list_str_t *comments=NULL;
 
   /* Comment for extension on further explanation. */
-  if( asprintf(&str, "See also: `%s' HDU of output with "
-               "`--checkdetection'", ( s0d1D2<2
+  if( asprintf(&str, "See also: '%s' HDU of output with "
+               "'--checkdetection'", ( s0d1D2<2
                                        ? "PSEUDOS-FOR-SN": "DILATED" ))<0 )
     error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_list_str_add(&comments, str, 0);
@@ -500,10 +537,10 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
   size_t tablen=num+1;
   gal_data_t *sn, *snind;
   int32_t *plabend, *indarr=NULL;
-  double ave, err, *xy, *brightness;
+  double ave, err, *pos, *brightness;
+  size_t ind, ndim=p->input->ndim, pcols=1+ndim;
   float s, ss, *f, *ff, *fs, *sky=p->sky->array;
-  size_t ind, ndim=p->input->ndim, xyncols=1+ndim;
-  size_t i, *area, counter=0, *dsize=p->input->dsize;
+  size_t i, j, *area, counter=0, *dsize=p->input->dsize;
   int32_t *plab = worklab->array, *dlab = s0d1D2 ? NULL : p->olabel->array;
   size_t *coord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
                                       "coord");
@@ -516,20 +553,20 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
     error(EXIT_FAILURE, 0, "%s: only a NaN value is recognized for blank "
           "floating point data types, the blank value is defined to be %f",
           __func__, GAL_BLANK_FLOAT32);
-  if(ndim!=2)
-    error(EXIT_FAILURE, 0, "%s: only 2D datasets are acceptable, your input "
-          "is %zu dimensions", __func__, ndim);
+  if(ndim!=2 && ndim!=3)
+    error(EXIT_FAILURE, 0, "%s: only 2D images or 3D datacubes are "
+          "acceptable, but input has %zu dimensions", __func__, ndim);
 
 
   /* Allocate all the necessary arrays, note that since we want to put each
      object's information into the same index, the number of allocated
-     spaces has to be `tablen=num+1'. */
+     spaces has to be 'tablen=num+1'. */
   area       = gal_pointer_allocate(GAL_TYPE_SIZE_T,  tablen, 1, __func__,
                                     "area");
   brightness = gal_pointer_allocate(GAL_TYPE_FLOAT64, tablen, 1, __func__,
                                      "brightness");
-  xy         = gal_pointer_allocate(GAL_TYPE_FLOAT64, xyncols*tablen, 1,
-                                     __func__, "xy");
+  pos        = gal_pointer_allocate(GAL_TYPE_FLOAT64, pcols*tablen, 1,
+                                    __func__, "pos");
   flag       = ( s0d1D2==0
                  ? gal_pointer_allocate(GAL_TYPE_UINT8, tablen, 1, __func__,
                                          "flag")
@@ -573,9 +610,11 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
           brightness[*plab] += ss;
           if( ss > 0.0f )  /* For calculatiing the approximate center, */
             {              /* necessary for calculating Sky and STD.   */
-              xy[*plab*xyncols  ] += ss;
-              xy[*plab*xyncols+1] += (double)coord[0] * ss;
-              xy[*plab*xyncols+2] += (double)coord[1] * ss;
+              pos[  *plab*pcols   ] += ss;
+              pos[  *plab*pcols+1 ] += (double)coord[0] * ss;
+              pos[  *plab*pcols+2 ] += (double)coord[1] * ss;
+              if(ndim==3)
+                pos[*plab*pcols+3 ] += (double)coord[2] * ss;
             }
         }
 
@@ -591,8 +630,8 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
     size_t i;
     for(i=1;i<num+1;++i)
       printf("%zu (%u): %-5zu %-13.3f %-13.3f %-13.3f %-13.3f\n", i, flag[i],
-             area[i], brightness[i], xy[i*xyncols], xy[i*xyncols+1],
-             xy[i*xyncols+2]);
+             area[i], brightness[i], pos[i*pcols], pos[i*pcols+1],
+             pos[i*pcols+2]);
   }
   */
 
@@ -624,11 +663,11 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
   for(i=1;i<tablen;++i)
     {
       ave=brightness[i]/area[i];
-      if( area[i]>p->snminarea && ave>0.0f && xy[i*xyncols]>0.0f )
+      if( area[i]>p->snminarea && ave>0.0f && pos[i*pcols]>0.0f )
         {
           /* Get the flux weighted center coordinates. */
-          coord[0]=GAL_DIMENSION_FLT_TO_INT( xy[i*xyncols+1]/xy[i*xyncols] );
-          coord[1]=GAL_DIMENSION_FLT_TO_INT( xy[i*xyncols+2]/xy[i*xyncols] );
+          for(j=0;j<ndim;++j)
+            coord[j]=GAL_DIMENSION_FLT_TO_INT(pos[i*pcols+j+1]/pos[i*pcols]);
 
           /* Get the Sky standard deviation on this tile. */
           err  = ((float *)(p->std->array))[
@@ -675,7 +714,7 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
 
 
   /* Clean up and return. */
-  free(xy);
+  free(pos);
   free(area);
   free(coord);
   free(brightness);
@@ -770,10 +809,10 @@ detection_pseudo_real(struct noisechiselparams *p)
       if( sn->size < p->minnumfalse)
         error(EXIT_FAILURE, 0, "only %zu pseudo-detections could be found "
               "over the sky region to estimate an S/N. This is less than "
-              "%zu (value to `--minnumfalse' option). Please adjust "
-              "parameters like `--dthresh', `--snminarea', or make sure "
+              "%zu (value to '--minnumfalse' option). Please adjust "
+              "parameters like '--dthresh', '--snminarea', or make sure "
               "that there actually is sufficient sky area after initial "
-              "detection. You can use `--checkdetection' to see every step "
+              "detection. You can use '--checkdetection' to see every step "
               "until this point", sn->size, p->minnumfalse);
 
 
@@ -891,12 +930,12 @@ detection_remove_false_initial(struct noisechiselparams *p,
                                           "newlabels");
 
   /* Find the new labels for all the existing labels. Recall that
-     `newlabels' was initialized to zero, so any label that is not given a
+     'newlabels' was initialized to zero, so any label that is not given a
      new label here will be automatically removed. After the first pixel of
      a label overlaps with dbyt[i], we don't need to check the rest of that
      object's pixels. At this point, tokeep is only binary: 0 or 1.
 
-     Note that the zeroth element of `tokeep' can also be non zero, this is
+     Note that the zeroth element of 'tokeep' can also be non zero, this is
      because the holes of the labeled regions might be filled during
      filling the holes, but have not been filled in the original labeled
      array. They are not important so you can just ignore them. */
@@ -928,7 +967,7 @@ detection_remove_false_initial(struct noisechiselparams *p,
      asked for growth, if the edges of the objects in the image are sharp
      enough, no growth will be necessary (and thus the labeled image won't
      be re-written during growth). So it is necessary to check for growth
-     here and later do it in `detection_quantile_expand'. */
+     here and later do it in 'detection_quantile_expand'. */
   p->numexpand=0;
   b=workbin->array;
   l=p->olabel->array;
@@ -988,7 +1027,7 @@ detection_remove_false_initial(struct noisechiselparams *p,
 
 /* Expand the initial detections based on the quantile threshold and then
    label the connected regions. If expansion is not possible, then return
-   the `GAL_BLANK_SIZET'.*/
+   the 'GAL_BLANK_SIZET'.*/
 static size_t
 detection_quantile_expand(struct noisechiselparams *p, gal_data_t *workbin)
 {
@@ -1149,32 +1188,39 @@ detection(struct noisechiselparams *p)
   if(!p->cp.quiet) gettimeofday(&t1, NULL);
   if(p->detgrowquant!=1.0f)
     {
+      /* Grow the detections and report the number of expanded (if
+         necessary). */
       num_expanded=detection_quantile_expand(p, workbin);
       if(num_expanded!=GAL_BLANK_SIZE_T)
-        num_true_initial=num_expanded;
-    }
-
-
-  /* Update the user on the progress, if necessary. */
-  if(!p->cp.quiet && p->detgrowquant!=1.0f && num_expanded!=GAL_BLANK_SIZE_T )
-    {
-      /* If the user hasn't asked for a labeled image, then don't confuse
-         them with the number of detections, just let them know that growth
-         is complete. */
-      if(p->label)
         {
-          if( asprintf(&msg, "%zu detections after growth to %.3f "
-                       "quantile.", num_true_initial, p->detgrowquant)<0 )
-            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+          /* Set the number of expanded objects. */
+          num_true_initial=num_expanded;
+
+          /* If not in quiet-mode, let the user know how its going. */
+          if(!p->cp.quiet)
+            {
+              /* If the user hasn't asked for a labeled image, then don't
+                 confuse them with the number of detections, just let them
+                 know that growth is complete. */
+              if(p->label)
+                {
+                  if( asprintf(&msg, "%zu detections after growth to %.3f "
+                               "quantile.", num_true_initial,
+                               p->detgrowquant)<0 )
+                    error(EXIT_FAILURE, 0, "%s: asprintf allocation",
+                          __func__);
+                }
+              else
+                {
+                  if( asprintf(&msg, "Growth to %.3f quantile complete.",
+                               p->detgrowquant)<0 )
+                    error(EXIT_FAILURE, 0, "%s: asprintf allocation",
+                          __func__);
+                }
+              gal_timing_report(&t1, msg, 2);
+              free(msg);
+            }
         }
-      else
-        {
-          if( asprintf(&msg, "Growth to %.3f quantile complete.",
-                       p->detgrowquant)<0 )
-            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
-        }
-      gal_timing_report(&t1, msg, 2);
-      free(msg);
     }
 
 
@@ -1214,8 +1260,8 @@ detection(struct noisechiselparams *p)
 
 
   /* p->binary was used to keep the initial pseudo-detection threshold. But
-     we don't need it any more, so we'll just free it and put the `workbin'
-     array in its place. Note that `workbin' has a map of all the detected
+     we don't need it any more, so we'll just free it and put the 'workbin'
+     array in its place. Note that 'workbin' has a map of all the detected
      objects, which is still necessary during NoiseChisel. */
   gal_data_free(p->binary);
   p->binary=workbin;
@@ -1228,7 +1274,7 @@ detection(struct noisechiselparams *p)
 
 
   /* If the user wanted to check the threshold and hasn't called
-     `continueaftercheck', then stop NoiseChisel. */
+     'continueaftercheck', then stop NoiseChisel. */
   if(p->detectionname && !p->continueaftercheck)
     ui_abort_after_check(p, p->detectionname, NULL,
                          "showing all detection steps");
