@@ -178,16 +178,17 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
       if(p->inputname)
         argp_error(state, "only one argument (input file) should be given");
       else
-        p->inputname=arg;
+        if(arg[0]!='\0') p->inputname=arg;
       break;
 
-
     /* This is an option, set its value. */
     default:
       return gal_options_set_from_key(key, arg, p->cp.poptions, &p->cp);
@@ -239,7 +240,7 @@ ui_read_check_only_options(struct noisechiselparams *p)
           "or '-t'). You have provided %.4f and %.4f for the former and "
           "latter, respectively", p->noerodequant, p->qthresh);
 
-  /* For the options that make tables, the table formation option is
+  /* For the options that make tables, the table format option is
      mandatory. */
   if( p->checksn && p->cp.tableformat==0 )
     error(EXIT_FAILURE, 0, "'--tableformat' is necessary with the "
@@ -256,7 +257,7 @@ ui_read_check_only_options(struct noisechiselparams *p)
       gal_checkset_check_file(p->kernelname);
 
       /* If its FITS, see if a HDU has been provided. */
-      if( gal_fits_name_is_fits(p->kernelname) && p->khdu==NULL )
+      if( gal_fits_file_recognized(p->kernelname) && p->khdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified for kernel. When the "
               "kernel is a FITS file, a HDU must also be specified. You "
               "can use the '--khdu' option and give it the HDU number "
@@ -271,7 +272,7 @@ ui_read_check_only_options(struct noisechiselparams *p)
       gal_checkset_check_file(p->widekernelname);
 
       /* If its FITS, see if a HDU has been provided. */
-      if( gal_fits_name_is_fits(p->widekernelname) && p->whdu==NULL )
+      if( gal_fits_file_recognized(p->widekernelname) && p->whdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified for the given wide kernel "
               "('%s'). When the wide kernel is a FITS file, a HDU must also "
               "be specified. You can use the '--whdu' option and give it the "
@@ -288,11 +289,31 @@ ui_read_check_only_options(struct noisechiselparams *p)
      '0.0001'. We were thus fully confused with the output (an extremely
      low value) and thought its a bug, while it wasn't! */
   if(p->snquant<0.1)
-    fprintf(stderr, "\nWARNING: Value of '--snquant' ('-c') is %g. Note "
-            "that this is not a contamination rate (where lower is "
-            "better), it is a purity rate (where higher is better). If you "
-            "intentionally asked for such a low purity level, please "
-            "ignore this warning\n\n", p->snquant);
+    error(EXIT_SUCCESS, 0, "\nWARNING: Value of '--snquant' ('-c') is "
+          "%g. Note that this is not a contamination rate (where lower "
+          "is better), it is a purity rate (where higher is better). If "
+          "you intentionally asked for such a low purity level, please "
+          "ignore this warning\n\n", p->snquant);
+
+  /* Make sure a reasonable value is given to '--meanmedqdiff'. */
+  if(p->meanmedqdiff>0.5)
+    error(EXIT_FAILURE, 0, "%g is not acceptable for '--meanmedqdiff'! "
+          "This option is the quantile-difference between the quantile "
+          "of the mean and 0.5 (the quantile of the median). Since the "
+          "range of quantile is from 0.0 to 1.0, the maximum difference "
+          "can therefore be 0.5. For more on this option, please see "
+          "the \"Quantifying signal in a tile\" section of the Gnuastro "
+          "book (with this command: 'info gnuastro \"Quantifying "
+          "signal in a tile\"'", p->meanmedqdiff);
+  if(p->meanmedqdiff>0.3 && p->cp.quiet==0)
+    error(EXIT_SUCCESS, 0, "WARNING: %g is very high for "
+          "'--meanmedqdiff'! This option is the quantile-difference "
+          "between the quantile of the mean and 0.5 (the quantile of "
+          "the median). For more on this option, please see the "
+          "\"Quantifying signal in a tile\" section of the Gnuastro "
+          "book (with this command: 'info gnuastro \"Quantifying "
+          "signal in a tile\"'. To suppress this warning, please use "
+          "the '--quiet' option", p->meanmedqdiff);
 }
 
 
@@ -309,7 +330,7 @@ ui_check_options_and_arguments(struct noisechiselparams *p)
       gal_checkset_check_file(p->inputname);
 
       /* If its FITS, see if a HDU has been provided. */
-      if( gal_fits_name_is_fits(p->inputname) && p->cp.hdu==NULL )
+      if( gal_fits_file_recognized(p->inputname) && p->cp.hdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified for input. When the input "
               "is a FITS file, a HDU must also be specified, you can use "
               "the '--hdu' ('-h') option and give it the HDU number "
@@ -406,21 +427,37 @@ ui_set_output_names(struct noisechiselparams *p)
 
 
 
+/* Prepare the kernel, either from a file, or from the default arrays
+   available in the headers. The default kernels were created as
+   follows. */
 static void
 ui_prepare_kernel(struct noisechiselparams *p)
 {
   float *f, *ff, *k;
+  size_t ndim=p->input->ndim;
 
-/* Impor the default kernel. */
+/* Import the default kernel. */
 #include "kernel-2d.h"
+#include "kernel-3d.h"
 
   /* If a kernel file is given, then use it. Otherwise, use the default
      kernel. */
   if(p->kernelname)
     {
+      /* Read the kernel. */
       if( strcmp(p->kernelname, UI_NO_CONV_KERNEL_NAME) )
-        p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
-                                           p->cp.minmapsize, p->cp.quietmmap);
+        {
+          /* Read the kernel into memory. */
+          p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
+                                             p->cp.minmapsize,
+                                             p->cp.quietmmap);
+
+          /* Make sure it has the same dimensions as the input. */
+          if( p->kernel->ndim != p->input->ndim )
+            error(EXIT_FAILURE, 0, "%s (hdu %s): is %zuD, however, %s (%s) "
+                  "is a %zuD dataset", p->kernelname, p->khdu,
+                  p->kernel->ndim, p->inputname, p->cp.hdu, p->input->ndim);
+        }
       else
         p->kernel=NULL;
     }
@@ -428,17 +465,17 @@ ui_prepare_kernel(struct noisechiselparams *p)
     {
       /* Allocate space for the kernel (we don't want to use the statically
          allocated array. */
-      p->kernel=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 2,
-                               kernel_2d_dsize, NULL, 0, p->cp.minmapsize,
-                               p->cp.quietmmap, NULL, NULL, NULL);
+      p->kernel=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, p->input->ndim,
+                               ndim==2 ? kernel_2d_dsize : kernel_3d_dsize,
+                               NULL, 0, p->cp.minmapsize, p->cp.quietmmap,
+                               NULL, NULL, NULL);
 
-      /* Now copy the staticly allocated array into it. */
-      k=p->kernel->array;
-      ff=(f=kernel_2d)+gal_dimension_total_size(2, p->kernel->dsize);
-      do *k++=*f; while(++f<ff);
+      /* Copy the staticly allocated default array into `p->kernel'. */
+      k = ndim==2 ? kernel_2d : kernel_3d;
+      ff = (f=p->kernel->array) + p->kernel->size;
+      do *f=*k++; while(++f<ff);
     }
 
-
   /* If a wide kernel is given, then read it into memory. Otherwise, just
      ignore it. */
   if(p->widekernelname)
@@ -541,7 +578,11 @@ ui_ngb_check(size_t value, char *optionname, size_t ndim)
               value, optionname);
       break;
     case 3:
-      error(EXIT_FAILURE, 0, "3D input data is not yet supported");
+      if(value!=6 && value!=18 && value!=26)
+        error(EXIT_FAILURE, 0, "%zu is not an acceptable value for "
+              "`--%s'. Acceptable values for 3D inputs are 6, 18 or 26",
+              value, optionname);
+      break;
     default:
       error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
             "problem. Dimension value %zu is not recognized.", __func__,
@@ -567,7 +608,8 @@ ui_preparations_read_input(struct noisechiselparams *p)
                                            NULL, GAL_TYPE_FLOAT32,
                                            p->cp.minmapsize,
                                            p->cp.quietmmap);
-  p->input->wcs = gal_wcs_read(p->inputname, p->cp.hdu, 0, 0,
+  p->input->wcs = gal_wcs_read(p->inputname, p->cp.hdu,
+                               p->cp.wcslinearmatrix, 0, 0,
                                &p->input->nwcs);
   p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
                                             p->input->dsize,
@@ -577,14 +619,15 @@ ui_preparations_read_input(struct noisechiselparams *p)
   if(p->input->name==NULL)
     gal_checkset_allocate_copy("INPUT", &p->input->name);
 
-  /* NoiseChisel currently only works on 2D datasets (images). */
-  if(p->input->ndim!=2)
-    error(EXIT_FAILURE, 0, "%s (hdu: %s) has %zu dimensions but NoiseChisel "
-          "can only operate on 2D datasets (images)", p->inputname, p->cp.hdu,
-          p->input->ndim);
-
-  /* Check the values of dimension-related options. */
+  /* Check the dimensions of the input. */
   ndim=p->input->ndim;
+  if(ndim!=2 && ndim!=3)
+    error(EXIT_FAILURE, 0, "%s: is a %zu dimensional dataset, only 2D "
+          "or 3D datasets are currently supported",
+          gal_fits_name_save_as_string(p->inputname, p->cp.hdu), ndim);
+
+  /* Check the neighbor options and if the given values correspond to the
+     input's dimensions. */
   ui_ngb_check(p->holengb, "holengb", ndim);
   ui_ngb_check(p->erodengb, "erodengb", ndim);
   ui_ngb_check(p->openingngb, "openingngb", ndim);
@@ -768,7 +811,10 @@ ui_read_check_inputs_setup(int argc, char *argv[],
                 printf("  - No convolution requested.\n");
             }
           else
-            printf("  - %s: FWHM=2 pixel Gaussian.\n",
+            printf(p->kernel->ndim==2
+                   ? "  - %s: FWHM=2 pixel Gaussian.\n"
+                   : "  - %s: FWHM=1.5 pixel Gaussian (half extent in "
+                   "3rd dimension).\n",
                    p->widekernelname ? "Sharp Kernel" : "Kernel");
         }
       if(p->widekernelname)
