@@ -186,16 +186,17 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
       if(p->inputname)
         argp_error(state, "only one argument (input file) should be given");
       else
-        p->inputname=arg;
+        if(arg[0]!='\0') p->inputname=arg;
       break;
 
-
     /* This is an option, set its value. */
     default:
       return gal_options_set_from_key(key, arg, p->cp.poptions, &p->cp);
@@ -429,14 +430,28 @@ ui_read_check_only_options(struct statisticsparams *p)
               "'--interpmetric' and '--interpnumngb' are mandatory when "
               "requesting Sky measurement ('--sky')");
 
-      /* If mode and median distance is a reasonable value. */
+      /* Make sure a reasonable value is given to '--meanmedqdiff'. */
       if(p->meanmedqdiff>0.5)
-        error(EXIT_FAILURE, 0, "%f not acceptable for '--meanmedqdiff'. It "
-              "cannot take values larger than 0.5 (quantile of median)",
-              p->meanmedqdiff);
+        error(EXIT_FAILURE, 0, "%g is not acceptable for '--meanmedqdiff'! "
+              "This option is the quantile-difference between the quantile "
+              "of the mean and 0.5 (the quantile of the median). Since the "
+              "range of quantile is from 0.0 to 1.0, the maximum difference "
+              "can therefore be 0.5. For more on this option, please see "
+              "the \"Quantifying signal in a tile\" section of the Gnuastro "
+              "book (with this command: 'info gnuastro \"Quantifying "
+              "signal in a tile\"'", p->meanmedqdiff);
+      if(p->meanmedqdiff>0.3 && p->cp.quiet==0)
+        error(EXIT_SUCCESS, 0, "WARNING: %g is very high for "
+              "'--meanmedqdiff'! This option is the quantile-difference "
+              "between the quantile of the mean and 0.5 (the quantile "
+              "of the median). For more on this option, please see the "
+              "\"Quantifying signal in a tile\" section of the Gnuastro "
+              "book (with this command: 'info gnuastro \"Quantifying "
+              "signal in a tile\"'. To suppress this warning, please use "
+              "the '--quiet' option", p->meanmedqdiff);
 
       /* If a kernel name has been given, we need the HDU. */
-      if(p->kernelname && gal_fits_name_is_fits(p->kernelname)
+      if(p->kernelname && gal_fits_file_recognized(p->kernelname)
          && p->khdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified for the kernel image. When "
               "A HDU is necessary for FITS files. You can use the '--khdu' "
@@ -540,7 +555,7 @@ ui_check_options_and_arguments(struct statisticsparams *p)
   if(p->inputname)
     {
       /* If input is FITS. */
-      if( (p->isfits=gal_fits_name_is_fits(p->inputname)) )
+      if( (p->isfits=gal_fits_file_recognized(p->inputname)) )
         {
           /* Check if a HDU is given. */
           if( p->cp.hdu==NULL )
@@ -587,9 +602,9 @@ static void
 ui_out_of_range_to_blank(struct statisticsparams *p)
 {
   size_t one=1;
-  unsigned char flags=GAL_ARITHMETIC_NUMOK;
-  unsigned char flagsor = ( GAL_ARITHMETIC_INPLACE
-                            | GAL_ARITHMETIC_NUMOK );
+  unsigned char flags=GAL_ARITHMETIC_FLAG_NUMOK;
+  unsigned char flagsor = ( GAL_ARITHMETIC_FLAG_INPLACE
+                            | GAL_ARITHMETIC_FLAG_NUMOK );
   gal_data_t *tmp, *tmp2, *cond_g=NULL, *cond_l=NULL, *cond, *blank, *ref;
 
 
@@ -953,13 +968,11 @@ ui_read_columns(struct statisticsparams *p)
 void
 ui_preparations(struct statisticsparams *p)
 {
-  unsigned char flagsor = ( GAL_ARITHMETIC_FREE
-                            | GAL_ARITHMETIC_INPLACE
-                            | GAL_ARITHMETIC_NUMOK );
   int keepinputdir=p->cp.keepinputdir;
   gal_data_t *check, *flag1, *flag2, *flag;
   struct gal_options_common_params *cp=&p->cp;
   struct gal_tile_two_layer_params *tl=&cp->tl;
+  unsigned char flagsor = GAL_ARITHMETIC_FLAGS_BASIC;
   char *checkbasename = p->cp.output ? p->cp.output : p->inputname;
 
   /* Change 'keepinputdir' based on if an output name was given. */
@@ -971,7 +984,8 @@ ui_preparations(struct statisticsparams *p)
       p->inputformat=INPUT_FORMAT_IMAGE;
       p->input=gal_array_read_one_ch(p->inputname, cp->hdu, NULL,
                                      cp->minmapsize, p->cp.quietmmap);
-      p->input->wcs=gal_wcs_read(p->inputname, cp->hdu, 0, 0,
+      p->input->wcs=gal_wcs_read(p->inputname, cp->hdu,
+                                 p->cp.wcslinearmatrix, 0, 0,
                                  &p->input->nwcs);
       p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
                                                 p->input->dsize,
