@@ -29,8 +29,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 
 #include <gnuastro/wcs.h>
+#include <gnuastro/type.h>
 
 #include <gnuastro-internal/checkset.h>
+#include <gnuastro-internal/arithmetic-set.h>
 
 #include "main.h"
 #include "arithmetic.h"
@@ -56,6 +58,8 @@ arithmetic_add_new_to_end(struct arithmetic_token **list)
 
   /* Initialize its elements. */
   node->next=NULL;
+  node->name_def=NULL;
+  node->name_use=NULL;
   node->constant=NULL;
   node->index=GAL_BLANK_SIZE_T;
   node->operator=GAL_ARITHMETIC_OP_INVALID;
@@ -84,6 +88,11 @@ arithmetic_token_free(struct arithmetic_token *list)
   struct arithmetic_token *tmp;
   while(list!=NULL)
     {
+      /* Free allocated elements first. */
+      if(list->name_def) free(list->name_def);
+      if(list->name_use) free(list->name_use);
+
+      /* Set the next list node, and free this one. */
       tmp=list->next;
       free(list);
       list=tmp;
@@ -119,21 +128,10 @@ arithmetic_operator_name(int operator)
   if(out==NULL)
     switch(operator)
       {
-      case ARITHMETIC_TABLE_OP_SIN: out="sin"; break;
-      case ARITHMETIC_TABLE_OP_COS: out="cos"; break;
-      case ARITHMETIC_TABLE_OP_TAN: out="tan"; break;
-      case ARITHMETIC_TABLE_OP_ASIN: out="asin"; break;
-      case ARITHMETIC_TABLE_OP_ACOS: out="acos"; break;
-      case ARITHMETIC_TABLE_OP_ATAN: out="atan"; break;
-      case ARITHMETIC_TABLE_OP_SINH: out="sinh"; break;
-      case ARITHMETIC_TABLE_OP_COSH: out="cosh"; break;
-      case ARITHMETIC_TABLE_OP_TANH: out="tanh"; break;
-      case ARITHMETIC_TABLE_OP_ASINH: out="asinh"; break;
-      case ARITHMETIC_TABLE_OP_ACOSH: out="acosh"; break;
-      case ARITHMETIC_TABLE_OP_ATANH: out="atanh"; break;
-      case ARITHMETIC_TABLE_OP_ATAN2: out="atan2"; break;
+      case ARITHMETIC_TABLE_OP_SET: out="set"; break;
       case ARITHMETIC_TABLE_OP_WCSTOIMG: out="wcstoimg"; break;
       case ARITHMETIC_TABLE_OP_IMGTOWCS: out="imgtowcs"; break;
+      case ARITHMETIC_TABLE_OP_DATETOSEC: out="date-to-sec"; break;
       case ARITHMETIC_TABLE_OP_DISTANCEFLAT: out="distance-flat"; break;
       case ARITHMETIC_TABLE_OP_DISTANCEONSPHERE: out="distance-on-sphere"; break;
       default:
@@ -161,7 +159,8 @@ arithmetic_init_wcs(struct tableparams *p, char *operator)
               "for the '%s' operator", operator);
 
       /* Read the WCS. */
-      p->wcs=gal_wcs_read(p->wcsfile, p->wcshdu, 0, 0, &p->nwcs);
+      p->wcs=gal_wcs_read(p->wcsfile, p->wcshdu, p->cp.wcslinearmatrix,
+                          0, 0, &p->nwcs);
       if(p->wcs==NULL)
         error(EXIT_FAILURE, 0, "%s (hdu: %s): no WCS could be read by "
               "WCSLIB", p->wcsfile, p->wcshdu);
@@ -182,36 +181,13 @@ arithmetic_set_operator(struct tableparams *p, char *string,
   /* Set the operator and number of operands. */
   if( op==GAL_ARITHMETIC_OP_INVALID )
     {
-      if(      !strcmp(string, "sin"))
-        { op=ARITHMETIC_TABLE_OP_SIN; *num_operands=0; }
-      else if( !strcmp(string, "cos"))
-        { op=ARITHMETIC_TABLE_OP_COS; *num_operands=0; }
-      else if( !strcmp(string, "tan"))
-        { op=ARITHMETIC_TABLE_OP_TAN; *num_operands=0; }
-      else if( !strcmp(string, "asin"))
-        { op=ARITHMETIC_TABLE_OP_ASIN; *num_operands=0; }
-      else if( !strcmp(string, "acos"))
-        { op=ARITHMETIC_TABLE_OP_ACOS; *num_operands=0; }
-      else if( !strcmp(string, "atan"))
-        { op=ARITHMETIC_TABLE_OP_ATAN; *num_operands=0; }
-      else if( !strcmp(string, "atan2"))
-        { op=ARITHMETIC_TABLE_OP_ATAN2; *num_operands=0; }
-      else if( !strcmp(string, "sinh"))
-        { op=ARITHMETIC_TABLE_OP_SINH; *num_operands=0; }
-      else if( !strcmp(string, "cosh"))
-        { op=ARITHMETIC_TABLE_OP_COSH; *num_operands=0; }
-      else if( !strcmp(string, "tanh"))
-        { op=ARITHMETIC_TABLE_OP_TANH; *num_operands=0; }
-      else if( !strcmp(string, "asinh"))
-        { op=ARITHMETIC_TABLE_OP_ASINH; *num_operands=0; }
-      else if( !strcmp(string, "acosh"))
-        { op=ARITHMETIC_TABLE_OP_ACOSH; *num_operands=0; }
-      else if( !strcmp(string, "atanh"))
-        { op=ARITHMETIC_TABLE_OP_ATANH; *num_operands=0; }
-      else if( !strcmp(string, "wcstoimg"))
+      /* Simple operators. */
+      if(      !strcmp(string, "wcstoimg"))
         { op=ARITHMETIC_TABLE_OP_WCSTOIMG; *num_operands=0; }
       else if( !strcmp(string, "imgtowcs"))
         { op=ARITHMETIC_TABLE_OP_IMGTOWCS; *num_operands=0; }
+      else if( !strcmp(string, "date-to-sec"))
+        { op=ARITHMETIC_TABLE_OP_DATETOSEC; *num_operands=0; }
       else if( !strcmp(string, "distance-flat"))
         { op=ARITHMETIC_TABLE_OP_DISTANCEFLAT; *num_operands=0; }
       else if( !strcmp(string, "distance-on-sphere"))
@@ -246,7 +222,7 @@ arithmetic_init(struct tableparams *p, struct arithmetic_token **arith,
   size_t one=1;
   uint8_t ntype;
   char *str, *delimiter=" \t";
-  struct arithmetic_token *node=NULL;
+  struct arithmetic_token *atmp, *node=NULL;
   char *token=NULL, *lasttoken=NULL, *saveptr;
 
   /* Parse all the given tokens. */
@@ -263,18 +239,40 @@ arithmetic_init(struct tableparams *p, struct arithmetic_token **arith,
         {
           /* Token is a single number.*/
           if( (num=gal_type_string_to_number(token, &ntype)) )
-            node->constant=gal_data_alloc(num, ntype, 1, &one, NULL, 0, -1, 1,
-                                          NULL, NULL, NULL);
+            node->constant=gal_data_alloc(num, ntype, 1, &one, NULL, 0,
+                                          -1, 1, NULL, NULL, NULL);
 
-          /* Token is a column operand (column number or name). */
+          /* This is a 'set-' operator. */
+          else if( !strncmp(token, GAL_ARITHMETIC_SET_PREFIX,
+                            GAL_ARITHMETIC_SET_PREFIX_LENGTH) )
+            {
+              node->num_operands=0;
+              node->operator=ARITHMETIC_TABLE_OP_SET;
+              gal_checkset_allocate_copy(token, &node->name_def);
+            }
+
+          /* Non-pre-defined situation.*/
           else
             {
-              str = ( (token[0]=='$' && isdigit(token[1]))
-                      ? &token[1]   /* Column number (starting with '$'). */
-                      : token );    /* Column name, just add it.          */
-              gal_list_str_add(toread, str, 1);
-              node->index=*totcalled;
-              *totcalled+=1;
+              /* See if this is an already defined name. */
+              for(atmp=*arith; atmp!=NULL; atmp=atmp->next)
+                if( atmp->name_def
+                    && strcmp(token,
+                              ( atmp->name_def
+                                + GAL_ARITHMETIC_SET_PREFIX_LENGTH) )==0 )
+                  gal_checkset_allocate_copy(token, &node->name_use);
+
+              /* If it wasn't found to be a pre-defined name, then its a
+                 column operand (column number or name). */
+              if(node->name_use==NULL)
+                {
+                  str = ( (token[0]=='$' && isdigit(token[1]))
+                          ? &token[1]   /* Column number (starting with '$'). */
+                          : token );    /* Column name, just add it.          */
+                  gal_list_str_add(toread, str, 1);
+                  node->index=*totcalled;
+                  *totcalled+=1;
+                }
             }
         }
 
@@ -310,9 +308,9 @@ arithmetic_indexs_final(struct tableparams *p, size_t *colmatch)
   for(tmp=p->outcols;tmp!=NULL;tmp=tmp->next)
     {
       /* If we are on an arithmetic operation. */
-      if(tmp->tokens)
+      if(tmp->arith)
         {
-          for(atmp=tmp->tokens;atmp!=NULL;atmp=atmp->next)
+          for(atmp=tmp->arith;atmp!=NULL;atmp=atmp->next)
             if(atmp->index!=GAL_BLANK_SIZE_T)
               {
                 /* Small sanity check. */
@@ -380,6 +378,14 @@ arithmetic_stack_pop(gal_data_t **stack, int operator, char *errormsg)
     error(EXIT_FAILURE, 0, "not enough operands for '%s'%s",
           arithmetic_operator_name(operator), errormsg?errormsg:"");
 
+  /* Arithmetic changes the contents of a dataset, so the old name (in the
+     FITS 'EXTNAME' keyword) must not be used beyond this
+     point. Furthermore, in Arithmetic, the 'name' element is used to
+     identify variables (with the 'set-' operator). */
+  if(out->name)    { free(out->name);    out->name=NULL;    }
+  if(out->unit)    { free(out->unit);    out->unit=NULL;    }
+  if(out->comment) { free(out->comment); out->comment=NULL; }
+
   /* Remove the 'next' element to break from the stack and return. */
   out->next=NULL;
   return out;
@@ -389,6 +395,53 @@ arithmetic_stack_pop(gal_data_t **stack, int operator, char *errormsg)
 
 
 
+/* Wrapper function to pop operands within the 'set-' operator. */
+static gal_data_t *
+arithmetic_stack_pop_wrapper_set(void *in)
+{
+  struct gal_arithmetic_set_params *tprm
+    = (struct gal_arithmetic_set_params *)in;
+  gal_data_t **stack=(gal_data_t **)tprm->params;
+  return arithmetic_stack_pop(stack, ARITHMETIC_TABLE_OP_SET, NULL);
+}
+
+
+
+
+
+/* For the 'set-' operator. */
+static int
+arithmetic_set_name_used_later(void *in, char *name)
+{
+  struct gal_arithmetic_set_params *p=(struct gal_arithmetic_set_params *)in;
+  struct arithmetic_token *tokens = (struct arithmetic_token *)(p->tokens);
+  struct arithmetic_token *token;
+
+  size_t counter=0;
+
+  /* If the name exists after the current token, then return 1. Note that
+     we have already separated the usage of names set with 'set-' in the
+     'name_use' element of the 'token' structure. */
+  for(token=tokens;token!=NULL;token=token->next)
+    {
+      if( token->name_use
+          && counter > p->tokencounter
+          && strcmp(token->name_use, name)==0 )
+        return 1;
+
+      /* Increment the counter (has to be after the check above because it
+         may not always get to the 'counter' variable). */
+      ++counter;
+    }
+
+  /* If we get to this point, it means that the name doesn't exist. */
+  return 0;
+}
+
+
+
+
+
 /* Set the converted column metadata. */
 static void
 arithmetic_update_metadata(gal_data_t *col, char *name, char *unit,
@@ -512,7 +565,7 @@ arithmetic_distance(struct tableparams *p, gal_data_t **stack, int operator)
   double *o, *a1, *a2, *b1, *b2;
   gal_data_t *a, *b, *tmp, *out;
   char *colname=NULL, *colcomment=NULL;
-  double (*distance_func)(double, double, double, double);
+  double (*distance_func)(double, double, double, double)=NULL;
 
   /* Pop the columns for point 'b'.*/
   tmp=arithmetic_stack_pop(stack, operator, NULL);
@@ -588,140 +641,76 @@ arithmetic_distance(struct tableparams *p, gal_data_t **stack, int operator)
 
 
 
+/* Convert the ISO date format to seconds since Unix time. */
 static void
-arithmetic_trig_hyper(struct tableparams *p, gal_data_t **stack,
-                      int operator)
+arithmetic_datetosec(struct tableparams *p, gal_data_t **stack,
+                     int operator)
 {
-  size_t i;
-  double *x=NULL, *y=NULL;
-  gal_data_t *in=NULL, *in2=NULL;
-  double pi=3.14159265358979323846264338327;
+  size_t i, v;
+  int64_t *iarr;
+  gal_data_t *out;
+  char *subsecstr=NULL;
+  double *darr, subsec=NAN;
 
-  /* Read the input columns as a 'double'. */
-  in=arithmetic_stack_pop(stack, operator, NULL);
-  in=gal_data_copy_to_new_type_free(in, GAL_TYPE_FLOAT64);
-  if(in->comment) { free(in->comment); in->comment=NULL; }
-  if(in->name)    { free(in->name);    in->name=NULL;    }
-  if(in->unit)    { free(in->unit);    in->unit=NULL;    }
-  x=in->array;
+  /* Input dataset. */
+  gal_data_t *in=arithmetic_stack_pop(stack, operator, NULL);
+  char **strarr=in->array;
 
-  /* Pop the second operand if necessary. */
-  if(operator==ARITHMETIC_TABLE_OP_ATAN2)
+  /* Output metadata. */
+  char *unit="sec";
+  char *name="UNIXSEC";
+  char *comment="Unix seconds (from 00:00:00 UTC, 1 January 1970)";
+
+  /* Make sure the input has a 'string' type. */
+  if(in->type!=GAL_TYPE_STRING)
+    error(EXIT_FAILURE, 0, "the operand given to 'date-to-sec' "
+          "should have a string type, but it is '%s'",
+          gal_type_name(in->type, 1));
+
+  /* Allocate the output dataset. */
+  out=gal_data_alloc(NULL, GAL_TYPE_INT64, 1, &in->size, NULL, 1,
+                     p->cp.minmapsize, p->cp.quietmmap, name, unit,
+                     comment);
+
+  /* Convert each input string (first try as an integer, assuming no
+     sub-second, or floating point precision).  */
+  iarr=out->array;
+  for(i=0; i<in->size; ++i)
     {
-      in2=arithmetic_stack_pop(stack, operator, NULL);
-      in2=gal_data_copy_to_new_type_free(in2, GAL_TYPE_FLOAT64);
-      y=in2->array;
+      v=gal_fits_key_date_to_seconds(strarr[i], &subsecstr,
+                                     &subsec);
+      iarr[i] = v==GAL_BLANK_SIZE_T ? GAL_BLANK_INT64 : v;
+      if(subsecstr) break;
     }
 
-  /* Parse the array and do the calculation in place. */
-  switch(operator)
+  /* If a sub-second string was present, then save the output as double
+     precision floating point. */
+  if(subsecstr)
     {
-      /* Single-operand operators. */
-    case ARITHMETIC_TABLE_OP_SIN:
-      for(i=0;i<in->size;++i) x[i]=sin( pi*x[i]/180.0f );
-      gal_checkset_allocate_copy("sin", &in->name);
-      gal_checkset_allocate_copy("ratio", &in->unit);
-      gal_checkset_allocate_copy("Sine of an angle.",
-                                 &in->comment);
-      break;
-    case ARITHMETIC_TABLE_OP_COS:
-      for(i=0;i<in->size;++i) x[i]=cos( pi*x[i]/180.0f );
-      gal_checkset_allocate_copy("cos", &in->name);
-      gal_checkset_allocate_copy("ratio", &in->unit);
-      gal_checkset_allocate_copy("Cosine of an angle.",
-                                 &in->comment);
-      break;
-    case ARITHMETIC_TABLE_OP_TAN:
-      for(i=0;i<in->size;++i) x[i]=tan( pi*x[i]/180.0f );
-      gal_checkset_allocate_copy("tan", &in->name);
-      gal_checkset_allocate_copy("ratio", &in->unit);
-      gal_checkset_allocate_copy("Tangent of an angle.",
-                                 &in->comment);
-      break;
-    case ARITHMETIC_TABLE_OP_ASIN:
-      for(i=0;i<in->size;++i) x[i]=asin(x[i])*180.0f/pi;
-      gal_checkset_allocate_copy("asin", &in->name);
-      gal_checkset_allocate_copy("deg", &in->unit);
-      gal_checkset_allocate_copy("Inverse sine of a value.",
-                                 &in->comment);
-      break;
-    case ARITHMETIC_TABLE_OP_ACOS:
-      for(i=0;i<in->size;++i) x[i]=acos(x[i])*180.0f/pi;
-      gal_checkset_allocate_copy("acos", &in->name);
-      gal_checkset_allocate_copy("deg", &in->unit);
-      gal_checkset_allocate_copy("Inverse cosine of a value.",
-                                 &in->comment);
-      break;
-    case ARITHMETIC_TABLE_OP_ATAN:
-      for(i=0;i<in->size;++i) x[i]=atan(x[i])*180.0f/pi;
-      gal_checkset_allocate_copy("atan", &in->name);
-      gal_checkset_allocate_copy("deg", &in->unit);
-      gal_checkset_allocate_copy("Inverse tangent of a value.",
-                                 &in->comment);
-      break;
-    case ARITHMETIC_TABLE_OP_SINH:
-      for(i=0;i<in->size;++i) x[i]=sinh(x[i]);
-      gal_checkset_allocate_copy("sinh", &in->name);
-      gal_checkset_allocate_copy("ratio", &in->unit);
-      gal_checkset_allocate_copy("Hyperbolic sine of a value.",
-                                 &in->comment);
-      break;
-    case ARITHMETIC_TABLE_OP_COSH:
-      for(i=0;i<in->size;++i) x[i]=cosh(x[i]);
-      gal_checkset_allocate_copy("cosh", &in->name);
-      gal_checkset_allocate_copy("ratio", &in->unit);
-      gal_checkset_allocate_copy("Hyperbolic cosine of a value.",
-                                 &in->comment);
-      break;
-    case ARITHMETIC_TABLE_OP_TANH:
-      for(i=0;i<in->size;++i) x[i]=tanh(x[i]);
-      gal_checkset_allocate_copy("tanh", &in->name);
-      gal_checkset_allocate_copy("ratio", &in->unit);
-      gal_checkset_allocate_copy("Hyperbolic tangent of a value.",
-                                 &in->comment);
-      break;
-    case ARITHMETIC_TABLE_OP_ASINH:
-      for(i=0;i<in->size;++i) x[i]=asinh(x[i]);
-      gal_checkset_allocate_copy("asinh", &in->name);
-      gal_checkset_allocate_copy("ratio", &in->unit);
-      gal_checkset_allocate_copy("Inverse hyperbolic sine of a value.",
-                                 &in->comment);
-      break;
-    case ARITHMETIC_TABLE_OP_ACOSH:
-      for(i=0;i<in->size;++i) x[i]=acosh(x[i]);
-      gal_checkset_allocate_copy("acosh", &in->name);
-      gal_checkset_allocate_copy("ratio", &in->unit);
-      gal_checkset_allocate_copy("Inverse hyperbolic cossine of a value.",
-                                 &in->comment);
-      break;
-    case ARITHMETIC_TABLE_OP_ATANH:
-      for(i=0;i<in->size;++i) x[i]=atanh(x[i]);
-      gal_checkset_allocate_copy("atanh", &in->name);
-      gal_checkset_allocate_copy("ratio", &in->unit);
-      gal_checkset_allocate_copy("Inverse hyperbolic tangent of a value.",
-                                 &in->comment);
-      break;
+      /* Free the initially allocated output (as an integer). */
+      free(subsecstr);
+      gal_data_free(out);
 
-    /* Two operand operators. */
-    case ARITHMETIC_TABLE_OP_ATAN2:
-      for(i=0;i<in->size;++i) x[i]=atan2(y[i], x[i])*180.0f/pi;
-      gal_checkset_allocate_copy("atan2", &in->name);
-      gal_checkset_allocate_copy("deg", &in->unit);
-      gal_checkset_allocate_copy("Inverse tangent of point (preserving "
-                                 "the quadrant)", &in->comment);
-      break;
+      /* Allocate a double-precision output. */
+      out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &in->size, NULL, 1,
+                         p->cp.minmapsize, p->cp.quietmmap, name, unit,
+                         comment);
 
-    /* Not recognized (a bug). */
-    default:
-      error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to fix "
-            "the problem. The code %d is not recognized as an operator "
-            "related to this function", __func__, PACKAGE_BUGREPORT,
-            operator);
+      /* Convert the date. */
+      darr=out->array;
+      for(i=0; i<in->size; ++i)
+        {
+          subsecstr=NULL;
+          v=gal_fits_key_date_to_seconds(strarr[i], &subsecstr,
+                                               &subsec);
+          darr[i] = v==GAL_BLANK_SIZE_T ? NAN : v;
+          if(subsecstr) { darr[i]+=subsec; free(subsecstr); }
+        }
     }
 
   /* Clean up and put the resulting calculation back on the stack. */
-  if(in2) gal_data_free(in2);
-  gal_list_data_add(stack, in);
+  if(in) gal_data_free(in);
+  gal_list_data_add(stack, out);
 }
 
 
@@ -775,36 +764,41 @@ arithmetic_placeholder_name(gal_data_t *col)
 
 
 static void
-arithmetic_operator_run(struct tableparams *p, gal_data_t **stack,
-                        int operator, size_t num_operands)
+arithmetic_operator_run(struct tableparams *p,
+                        struct arithmetic_token *token,
+                        struct gal_arithmetic_set_params *setprm,
+                        gal_data_t **stack)
 {
+  int flags=GAL_ARITHMETIC_FLAGS_BASIC;
   gal_data_t *d1=NULL, *d2=NULL, *d3=NULL;
-  int flags = ( GAL_ARITHMETIC_INPLACE | GAL_ARITHMETIC_FREE
-                | GAL_ARITHMETIC_NUMOK );
+
+  /* Set the operating-mode flags if necessary. */
+  if(p->cp.quiet) flags |= GAL_ARITHMETIC_FLAG_QUIET;
+  if(p->envseed)  flags |= GAL_ARITHMETIC_FLAG_ENVSEED;
 
   /* When 'num_operands!=0', the operator is in the library. */
-  if(num_operands)
+  if(token->num_operands)
     {
       /* Pop the necessary number of operators. Note that the
          operators are poped from a linked list (which is
          last-in-first-out). So for the operators which need a
          specific order, the first poped operand is actally the
          last (right most, in in-fix notation) input operand.*/
-      switch(num_operands)
+      switch(token->num_operands)
         {
         case 1:
-          d1=arithmetic_stack_pop(stack, operator, NULL);
+          d1=arithmetic_stack_pop(stack, token->operator, NULL);
           break;
 
         case 2:
-          d2=arithmetic_stack_pop(stack, operator, NULL);
-          d1=arithmetic_stack_pop(stack, operator, NULL);
+          d2=arithmetic_stack_pop(stack, token->operator, NULL);
+          d1=arithmetic_stack_pop(stack, token->operator, NULL);
           break;
 
         case 3:
-          d3=arithmetic_stack_pop(stack, operator, NULL);
-          d2=arithmetic_stack_pop(stack, operator, NULL);
-          d1=arithmetic_stack_pop(stack, operator, NULL);
+          d3=arithmetic_stack_pop(stack, token->operator, NULL);
+          d2=arithmetic_stack_pop(stack, token->operator, NULL);
+          d1=arithmetic_stack_pop(stack, token->operator, NULL);
           break;
 
         case -1:
@@ -817,7 +811,8 @@ arithmetic_operator_run(struct tableparams *p, gal_data_t **stack,
           error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
                 "the problem. '%zu' is not recognized as an operand "
                 "counter (with '%s')", __func__, PACKAGE_BUGREPORT,
-                num_operands, arithmetic_operator_name(operator));
+                token->num_operands,
+                arithmetic_operator_name(token->operator));
         }
 
       /* Run the arithmetic operation. Note that 'gal_arithmetic' is a
@@ -825,7 +820,7 @@ arithmetic_operator_run(struct tableparams *p, gal_data_t **stack,
          arguments it uses depend on the operator. In other words, when the
          operator doesn't need three operands, the extra arguments will be
          ignored. */
-      gal_list_data_add(stack, gal_arithmetic(operator, p->cp.numthreads,
+      gal_list_data_add(stack, gal_arithmetic(token->operator, p->cp.numthreads,
                                               flags, d1, d2, d3) );
 
       /* Reset the meta-data for the element that was just put on the
@@ -836,38 +831,30 @@ arithmetic_operator_run(struct tableparams *p, gal_data_t **stack,
   /* This operator is specific to this program (Table). */
   else
     {
-      switch(operator)
+      switch(token->operator)
         {
-        case ARITHMETIC_TABLE_OP_SIN:
-        case ARITHMETIC_TABLE_OP_COS:
-        case ARITHMETIC_TABLE_OP_TAN:
-        case ARITHMETIC_TABLE_OP_ASIN:
-        case ARITHMETIC_TABLE_OP_ACOS:
-        case ARITHMETIC_TABLE_OP_ATAN:
-        case ARITHMETIC_TABLE_OP_SINH:
-        case ARITHMETIC_TABLE_OP_COSH:
-        case ARITHMETIC_TABLE_OP_TANH:
-        case ARITHMETIC_TABLE_OP_ASINH:
-        case ARITHMETIC_TABLE_OP_ACOSH:
-        case ARITHMETIC_TABLE_OP_ATANH:
-        case ARITHMETIC_TABLE_OP_ATAN2:
-          arithmetic_trig_hyper(p, stack, operator);
-          break;
-
         case ARITHMETIC_TABLE_OP_WCSTOIMG:
         case ARITHMETIC_TABLE_OP_IMGTOWCS:
-          arithmetic_wcs(p, stack, operator);
+          arithmetic_wcs(p, stack, token->operator);
+          break;
+
+        case ARITHMETIC_TABLE_OP_DATETOSEC:
+          arithmetic_datetosec(p, stack, token->operator);
           break;
 
         case ARITHMETIC_TABLE_OP_DISTANCEFLAT:
         case ARITHMETIC_TABLE_OP_DISTANCEONSPHERE:
-          arithmetic_distance(p, stack, operator);
+          arithmetic_distance(p, stack, token->operator);
+          break;
+
+        case ARITHMETIC_TABLE_OP_SET:
+          gal_arithmetic_set_name(setprm, token->name_def);
           break;
 
         default:
           error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
                 "fix the problem. The operator code %d is not recognized",
-                __func__, PACKAGE_BUGREPORT, operator);
+                __func__, PACKAGE_BUGREPORT, token->operator);
         }
     }
 }
@@ -880,16 +867,28 @@ arithmetic_operator_run(struct tableparams *p, gal_data_t **stack,
 static void
 arithmetic_reverse_polish(struct tableparams *p, struct column_pack *outpack)
 {
-  gal_data_t *single, *stack=NULL;
   struct arithmetic_token *token;
+  gal_data_t *single, *stack=NULL;
+  struct gal_arithmetic_set_params setprm={0};
+
+  /* Initialize the arithmetic functions/pointers. */
+  setprm.params=&stack;
+  setprm.tokens=outpack->arith;
+  setprm.pop=arithmetic_stack_pop_wrapper_set;
+  setprm.used_later=arithmetic_set_name_used_later;
 
   /* Go through all the tokens given to this element. */
-  for(token=outpack->tokens;token!=NULL;token=token->next)
+  for(token=outpack->arith;token!=NULL;token=token->next)
     {
       /* We are on an operator. */
       if(token->operator!=GAL_ARITHMETIC_OP_INVALID)
-        arithmetic_operator_run(p, &stack, token->operator,
-                                token->num_operands);
+        arithmetic_operator_run(p, token, &setprm, &stack);
+
+      /* We are on a named variable. */
+      else if( token->name_use )
+        gal_list_data_add(&stack,
+                          gal_arithmetic_set_copy_named(&setprm,
+                                                        token->name_use));
 
       /* Constant number: just put it ontop of the stack. */
       else if(token->constant)
@@ -907,6 +906,9 @@ arithmetic_reverse_polish(struct tableparams *p, struct column_pack *outpack)
         error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
               "fix the problem. The token can't be identified as an "
               "operator, constant or column", __func__, PACKAGE_BUGREPORT);
+
+      /* Increment the token counter. */
+      ++setprm.tokencounter;
     }
 
   /* Put everything that remains in the stack (reversed) into the final
@@ -965,16 +967,16 @@ arithmetic_operate(struct tableparams *p)
   /* From now on, we will be looking for columns from the index in
      'colarray', so to keep things clean, we'll set all the 'next' elements
      to NULL. */
-  for(i=0;i<p->numcolarray;++i) p->colarray[i]->next=NULL;
+  for(i=0; i<p->numcolarray; ++i) p->colarray[i]->next=NULL;
 
   /* We'll also reset the output table pointer, to fill it in as we
      progress. */
   p->table=NULL;
 
   /* Go over each package of columns. */
-  for(outpack=p->outcols;outpack!=NULL;outpack=outpack->next)
+  for(outpack=p->outcols; outpack!=NULL; outpack=outpack->next)
     {
-      if(outpack->tokens)
+      if(outpack->arith)
         arithmetic_reverse_polish(p, outpack);
       else
         {
