@@ -28,8 +28,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 #include <string.h>
 
+#include <gnuastro/wcs.h>
 #include <gnuastro/git.h>
 #include <gnuastro/txt.h>
+#include <gnuastro/ds9.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/list.h>
 #include <gnuastro/data.h>
@@ -476,6 +478,54 @@ gal_options_read_searchin(struct argp_option *option, char *arg,
 
 
 
+void *
+gal_options_read_wcslinearmatrix(struct argp_option *option, char *arg,
+                                 char *filename, size_t lineno, void *junk)
+{
+  char *str;
+  uint8_t value=GAL_WCS_LINEAR_MATRIX_INVALID;
+  if(lineno==-1)
+    {
+      /* The output must be an allocated string (will be 'free'd later). */
+      value=*(uint8_t *)(option->value);
+      switch(value)
+        {
+        case GAL_WCS_LINEAR_MATRIX_PC: gal_checkset_allocate_copy("pc", &str);
+          break;
+        case GAL_WCS_LINEAR_MATRIX_CD: gal_checkset_allocate_copy("cd", &str);
+          break;
+        default:
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' "
+                "to fix the problem. %u is not a recognized WCS rotation "
+                "matrix code", __func__, PACKAGE_BUGREPORT, value);
+        }
+      return str;
+    }
+  else
+    {
+      /* If the option is already set, just return. */
+      if(option->set) return NULL;
+
+      /* Read the value. */
+      if(      !strcmp(arg, "pc") ) value = GAL_WCS_LINEAR_MATRIX_PC;
+      else if( !strcmp(arg, "cd") ) value = GAL_WCS_LINEAR_MATRIX_CD;
+      else
+        error_at_line(EXIT_FAILURE, 0, filename, lineno, "'%s' (value "
+                      "to '%s' option) couldn't be recognized as a known "
+                      "WCS rotation matrix. Acceptable values are 'pc' "
+                      "or 'cd'", arg, option->name);
+      *(uint8_t *)(option->value)=value;
+
+      /* For no un-used variable warning. This function doesn't need the
+         pointer.*/
+      return junk=NULL;
+    }
+}
+
+
+
+
+
 void *
 gal_options_read_tableformat(struct argp_option *option, char *arg,
                              char *filename, size_t lineno, void *junk)
@@ -1338,8 +1388,9 @@ gal_options_parse_name_and_float64s(struct argp_option *option, char *arg,
 /* Parse strings like this: 'num1,num2:num3,n4:num5,num6' and return it as
    a data container array: all elements are simply placed after each other
    in the array. */
-static gal_data_t *
-options_parse_colon_sep_csv(char *instring, char *filename, size_t lineno)
+gal_data_t *
+gal_options_parse_colon_sep_csv_raw(char *instring, char *filename,
+                                    size_t lineno)
 {
   char *tailptr;
   gal_data_t *out;
@@ -1479,8 +1530,13 @@ gal_options_parse_colon_sep_csv(struct argp_option *option, char *arg,
         error_at_line(EXIT_FAILURE, 0, filename, lineno, "no value "
                       "given to '--%s'", option->name);
 
-      /* Parse the desired format and put it in this option's pointer. */
-      dataset=options_parse_colon_sep_csv(arg, filename, lineno);
+      /* Check if the argument is a string (it contains a ':' or a ',') or
+         a filename. Then parse the desired format and put it in this
+         option's pointer. */
+      if( strchr(arg, ',')==NULL && strchr(arg, ':')==NULL )
+        dataset=gal_ds9_reg_read_polygon(arg);
+      else
+        dataset=gal_options_parse_colon_sep_csv_raw(arg, filename, lineno);
 
       /* Add the given dataset to the end of an existing dataset. */
       existing = *(gal_data_t **)(option->value);
@@ -1526,13 +1582,11 @@ options_sanity_check(struct argp_option *option, char *arg,
 {
   size_t dsize=1;
   char *message=NULL;
+  int mcflag=GAL_ARITHMETIC_FLAGS_BASIC;
   int operator1=GAL_ARITHMETIC_OP_INVALID;
   int operator2=GAL_ARITHMETIC_OP_INVALID;
   int multicheckop=GAL_ARITHMETIC_OP_INVALID;
   gal_data_t *value, *ref1=NULL, *ref2=NULL, *check1, *check2;
-  int mcflag = ( GAL_ARITHMETIC_NUMOK
-                 | GAL_ARITHMETIC_FREE
-                 | GAL_ARITHMETIC_INPLACE );
 
   /* Currently, this function is only for numeric types, so if the value is
      string type, or its 'range' field is 'GAL_OPTIONS_RANGE_ANY', then
@@ -1670,10 +1724,12 @@ options_sanity_check(struct argp_option *option, char *arg,
      'GAL_ARITHMETIC_INPLACE' flags. But we will do this when there are
      multiple checks so from the two check data structures, we only have
      one remaining. */
-  check1=gal_arithmetic(operator1, 1, GAL_ARITHMETIC_NUMOK, value, ref1);
+  check1=gal_arithmetic(operator1, 1, GAL_ARITHMETIC_FLAG_NUMOK,
+                        value, ref1);
   if(ref2)
     {
-      check2=gal_arithmetic(operator2, 1, GAL_ARITHMETIC_NUMOK, value, ref2);
+      check2=gal_arithmetic(operator2, 1, GAL_ARITHMETIC_FLAG_NUMOK,
+                            value, ref2);
       check1=gal_arithmetic(multicheckop, 1, mcflag, check1, check2);
     }
 
@@ -2135,7 +2191,7 @@ options_lastconfig_has_been_called(struct argp_option *coptions)
 
 static void
 options_parse_file(char *filename,  struct gal_options_common_params *cp,
-                   int enoent_abort)
+                   int warning)
 {
   FILE *fp;
   char *line, *name, *arg;
@@ -2152,7 +2208,13 @@ options_parse_file(char *filename,  struct gal_options_common_params *cp,
      ignore the configuration file and return. */
   errno=0;
   fp=fopen(filename, "r");
-  if(fp==NULL) return;
+  if(fp==NULL)
+    {
+      /* Print a warning  */
+      if(warning && cp->quiet==0)
+        error(EXIT_SUCCESS, errno, "%s", filename);
+      return;
+    }
 
 
   /* If necessary, print the configuration file name. */
