@@ -331,3 +331,58 @@ gal_units_degree_to_dec(double decimal, int usecolon)
   /* Return the final string. */
   return dec;
 }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+/**********************************************************************/
+/****************          Flux conversions           *****************/
+/**********************************************************************/
+
+/* Convert counts to magnitude using the given zeropoint. */
+double
+gal_units_counts_to_mag(double counts, double zeropoint)
+{
+  return ( counts > 0.0f
+           ? ( -2.5f * log10(counts) + zeropoint )
+           : NAN );
+}
+
+
+
+
+
+/* Convert magnitude to counts using the given zeropoint. */
+double
+gal_units_mag_to_counts(double mag, double zeropoint)
+{
+  return pow(10, (mag - zeropoint)/(-2.5f));
+}
+
+
+
+
+/* Convert Pixel values to Janskys with an AB-magnitude based
+   zero-point. See the "Brightness, Flux, Magnitude and Surface
+   brightness". */
+double
+gal_units_counts_to_jy(double counts, double zeropoint_ab)
+{
+  return counts * 3631 * pow(10, -1 * zeropoint_ab / 2.5);
+}
